import { DriverErrorTelemetryProps } from '@fluidframework/driver-utils';
import { IFluidErrorBase } from '@fluidframework/telemetry-utils';
import { IOdspErrorAugmentations } from '@fluidframework/odsp-driver-definitions';
import type { ITelemetryBaseProperties } from '@fluidframework/core-interfaces';
import { LoggingError } from '@fluidframework/telemetry-utils';
import { OdspError } from '@fluidframework/odsp-driver-definitions';

/**
 * Issue the requestCallback, providing the proper auth header based on authRequestInfo,
 * and retrying with a refreshed token if necessary.
 * @internal
 */
export declare function authRequestWithRetry(authRequestInfo: IOdspAuthRequestInfo, requestCallback: (config: RequestInit) => Promise<Response>): Promise<Response>;

/**
 * @internal
 */
export declare function createOdspNetworkError(errorMessage: string, statusCode: number, retryAfterSeconds?: number, response?: Response, responseText?: string, props?: ITelemetryBaseProperties): IFluidErrorBase & OdspError;

/**
 * @internal
 */
export declare function enrichOdspError(error: IFluidErrorBase & OdspError, response?: Response, facetCodes?: string[], props?: ITelemetryBaseProperties): IFluidErrorBase & OdspError;

/**
 * @internal
 */
export declare const fetchIncorrectResponse = 712;

/**
 * Fetch an access token and refresh token from AAD
 * @param server - The server to auth against
 * @param scope - The desired oauth scope
 * @param clientConfig - Info about this client's identity
 * @param credentials - Credentials authorizing the client for the requested token
 * @internal
 */
export declare function fetchTokens(server: string, scope: string, clientConfig: IPublicClientConfig, credentials: TokenRequestCredentials): Promise<IOdspTokens>;

/**
 * @internal
 */
export declare function getAadTenant(server: string): string;

/**
 * @internal
 */
export declare function getAadUrl(server: string): string;

/**
 * @internal
 */
export declare function getAsync(url: string, authRequestInfo: IOdspAuthRequestInfo): Promise<Response>;

/**
 * @internal
 */
export declare function getChildrenByDriveItem(driveItem: IOdspDriveItem, server: string, authRequestInfo: IOdspAuthRequestInfo): Promise<IOdspDriveItem[]>;

/**
 * @alpha
 */
export declare function getDriveId(server: string, account: string, library: string | undefined, authRequestInfo: IOdspAuthRequestInfo): Promise<string>;

/**
 * @internal
 */
export declare function getDriveItemByRootFileName(server: string, account: string | undefined, path: string, authRequestInfo: IOdspAuthRequestInfo, create: boolean, driveId?: string): Promise<IOdspDriveItem>;

/**
 * @internal
 */
export declare function getDriveItemByServerRelativePath(server: string, serverRelativePath: string, authRequestInfo: IOdspAuthRequestInfo, create: boolean): Promise<IOdspDriveItem>;

/**
 * @internal
 */
export declare function getDriveItemFromDriveAndItem(server: string, drive: string, item: string, authRequestInfo: IOdspAuthRequestInfo): Promise<IOdspDriveItem>;

/**
 * @internal
 */
export declare function getFetchTokenUrl(server: string): string;

/**
 * @internal
 */
export declare function getLoginPageUrl(server: string, clientConfig: IPublicClientConfig, scope: string, odspAuthRedirectUri: string): string;

/**
 * @internal
 */
export declare const getOdspRefreshTokenFn: (server: string, clientConfig: IPublicClientConfig, tokens: IOdspTokens) => () => Promise<string>;

/**
 * @alpha
 */
export declare const getOdspScope: (server: string) => string;

/**
 * @internal
 */
export declare const getPushRefreshTokenFn: (server: string, clientConfig: IPublicClientConfig, tokens: IOdspTokens) => () => Promise<string>;

/**
 * @internal
 */
export declare const getRefreshTokenFn: (scope: string, server: string, clientConfig: IPublicClientConfig, tokens: IOdspTokens) => () => Promise<string>;

/**
 * @internal
 */
export declare function getServer(tenantId: string): string;

/**
 * @internal
 */
export declare function getSiteUrl(server: string): string;

/**
 * @internal
 */
export declare function getSPOAndGraphRequestIdsFromResponse(headers: {
    get: (id: string) => string | undefined | null;
}): ITelemetryBaseProperties;

/**
 * @internal
 */
export declare function hasFacetCodes(x: any): x is Pick<IOdspErrorAugmentations, "facetCodes">;

/**
 * @alpha
 */
export declare interface IOdspAuthRequestInfo {
    accessToken: string;
    refreshTokenFn?: () => Promise<string>;
}

/**
 * @internal
 */
export declare interface IOdspDriveItem {
    path: string;
    name: string;
    driveId: string;
    itemId: string;
    isFolder: boolean;
}

/**
 * @internal
 */
export declare interface IOdspTokens {
    readonly accessToken: string;
    readonly refreshToken: string;
}

/**
 * Configuration for a public client.
 * @internal
 */
export declare interface IPublicClientConfig {
    clientId: string;
}

/**
 * @internal
 */
export declare function isOdspHostname(server: string): boolean;

/**
 * @internal
 */
export declare function isPushChannelHostname(server: string): boolean;

/**
 * Empirically-based model of error responses from ODSP
 *
 * @internal
 */
export declare interface OdspErrorResponse {
    error: OdspErrorResponseInnerError & {
        message: string;
    };
}

/**
 * Empirically-based model of error response inner error from ODSP
 *
 * @internal
 */
export declare interface OdspErrorResponseInnerError {
    code?: string;
    innerError?: OdspErrorResponseInnerError;
}

/**
 * Error encapsulating the error response from ODSP containing the redirect location when a resource has moved
 *
 * @internal
 */
export declare class OdspRedirectError extends LoggingError implements IFluidErrorBase {
    readonly redirectLocation: string | undefined;
    readonly errorType: "fileNotFoundOrAccessDeniedError";
    readonly canRetry = false;
    constructor(message: string, redirectLocation: string | undefined, props: DriverErrorTelemetryProps);
}

/**
 * @internal
 */
export declare const OdspServiceReadOnlyErrorCode = "serviceReadOnly";

/**
 * @internal
 */
export declare function parseFacetCodes(errorResponse: OdspErrorResponse): string[];

/**
 * @internal
 */
export declare function postAsync(url: string, body: any, authRequestInfo: IOdspAuthRequestInfo): Promise<Response>;

/**
 * @alpha
 */
export declare const pushScope = "offline_access https://pushchannel.1drv.ms/PushChannel.ReadWrite.All";

/**
 * @internal
 */
export declare function putAsync(url: string, authRequestInfo: IOdspAuthRequestInfo): Promise<Response>;

/**
 * Fetch fresh tokens.
 * @param server - The server to auth against
 * @param scope - The desired oauth scope
 * @param clientConfig - Info about this client's identity
 * @param tokens - The tokens object provides the refresh token for the request
 *
 * @returns The tokens object with refreshed tokens.
 * @internal
 */
export declare function refreshTokens(server: string, scope: string, clientConfig: IPublicClientConfig, tokens: IOdspTokens): Promise<IOdspTokens>;

/**
 * Throws network error - an object with a bunch of network related properties
 * @internal
 */
export declare function throwOdspNetworkError(errorMessage: string, statusCode: number, response: Response, responseText?: string, props?: ITelemetryBaseProperties): never;

/**
 * @internal
 */
export declare type TokenRequestCredentials = {
    grant_type: "authorization_code";
    code: string;
    redirect_uri: string;
} | {
    grant_type: "refresh_token";
    refresh_token: string;
} | {
    grant_type: "password";
    username: string;
    password: string;
};

/**
 * @internal
 */
export declare function tryParseErrorResponse(response: string | undefined): {
    success: true;
    errorResponse: OdspErrorResponse;
} | {
    success: false;
};

/**
 * @internal
 */
export declare function unauthPostAsync(url: string, body: any): Promise<Response>;

export { }
